/*
	WordWrap.cmd
	WordWrap v1.3 by Don Eitner, 2001

	Reformats text files and reflows them to a line length you specify.

	This code is neither supported nor under warranty.  Feel free to
	examine and modify this script for your own purposes.  See the
	included readme.txt for additional information.
*/

CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
CALL SysLoadFuncs

CALL AnsiSetup

PARSE ARG CmdLine
PARSE VAR CmdLine CL1 CmdLine
CL1 = Translate(CL1)

Outline = ''
OutFile = ''

CALL ParseInput

OutForm = Translate(OutForm)

SELECT
	WHEN Translate(EOL) = 'UNIX' | Translate(EOL) = 'LF' | Translate(EOL) = '0A' THEN
		DO
			EOL2 = 'UNIX'
			EOL = '0a'x
		END
	WHEN Translate(EOL) = 'DOS' | Translate(EOL) = 'CRLF' | Translate(EOL) = '0D0A' THEN
		DO
			EOL2 = 'DOS'
			EOL = '0d0a'x
		END
	WHEN Translate(EOL) = 'MAC' | Translate(EOL) = 'CR' | Translate(EOL) = '0D' THEN
		DO
			EOL2 = 'MAC'
			EOL = '0d'x
		END
	OTHERWISE
		EOL = '0d0a'x
END

/* Ensure that input line is valid */
IF (DATATYPE(Margin) <> 'NUM') THEN
	DO
		SAY 'You must provide a line length (in characters).'
		EXIT
	END
ELSE
	SELECT
		WHEN Margin < 0 THEN   /* Cannot do this */
			SAY 'Line length must be a whole number greater than or equal to zero.'
		WHEN Margin = 0 THEN   /* Only reformat, don't wrap lines */
			DO
				CALL Prep
				CALL Reformat
				CALL CleanUp
			END
		OTHERWISE   /* Reformat and wrap lines */
			DO
				CALL Prep
				CALL Reformat
				CALL Wrap
				CALL CleanUp
			END
	END

EXIT


AnsiSetup:

	Esc=d2c(27)d2c(91)

	/* Attributes */
	Hig=Esc'1m'
	Non=Esc'0m'

	/* Foreground Colors */
	Bla=Esc'30m'
	Red=Esc'31m'
	Gre=Esc'32m'
	Bro=Esc'33m'
	Blu=Esc'34m'
	Pur=Esc'35m'
	Tur=Esc'36m'
	Whi=Esc'37m'
	LBla=Hig||Bla
	LRed=Hig||Red
	LGre=Hig||Gre
	LBro=Hig||Bro
	LBlu=Hig||Blu
	LPur=Hig||Pur
	LTur=Hig||Tur
	LWhi=Hig||Whi

	/* Background Colors */
	BBla=Esc'40m'
	BRed=Esc'41m'
	BGre=Esc'42m'
	BBro=Esc'43m'
	BBlu=Esc'44m'
	BCya=Esc'45m'
	BTur=Esc'46m'
	BWhi=Esc'47m'

RETURN


ParseInput:

	Again:

	IF (CL1 = '/M') THEN
		DO
			PARSE VAR CmdLine CL1 CmdLine
			Margin = CL1
			PARSE VAR CmdLine CL1 CmdLine
			CL1 = Translate(CL1)
			SIGNAL Again
		END

	IF (CL1 = '/F') THEN
		DO
			PARSE VAR CmdLine CL1 CmdLine
			OutForm = CL1
			PARSE VAR CmdLine CL1 CmdLine
			CL1 = Translate(CL1)
			SIGNAL Again
		END

	IF (CL1 = '/E') THEN
		DO
			PARSE VAR CmdLine CL1 CmdLine
			EOL = CL1
			PARSE VAR CmdLine CL1 CmdLine
			CL1 = Translate(CL1)
			SIGNAL Again
		END

	IF (CL1 = '/I') THEN
		DO
			IF Left(CmdLine, 1) = '"' THEN
				DO
					PARSE VAR CmdLine WITH '"' CL1 '"' CmdLine
					OrigFile = CL1
				END
			ELSE
				DO
					PARSE VAR CmdLine CL1 CmdLine
					OrigFile = CL1
				END
			PARSE VAR CmdLine CL1 CmdLine
			CL1 = Translate(CL1)
			SIGNAL Again
		END

	IF (CL1 = '/O') THEN
		DO
			IF Left(CmdLine, 1) = '"' THEN
				DO
					PARSE VAR CmdLine WITH '"' CL1 '"' CmdLine
					OutFile = CL1
				END
			ELSE
				DO
					PARSE VAR CmdLine CL1 CmdLine
					OutFile = CL1
				END
			PARSE VAR CmdLine CL1 CmdLine
			CL1 = Translate(CL1)
			SIGNAL Again
		END

	IF (CL1 = '/?') THEN
		CALL GiveHelp

RETURN


GiveHelp:

	CALL SysCls
	SAY LTur||'WordWrap v1.3, (c) 2001 by Don Eitner'||Non
	SAY ''
	SAY LGre||'Usage:'||Non
	SAY '  WordWrap /M Margin /I InFile /O OutFile [options]'
	SAY ''
	SAY ' Margin must be an integer.  Margin of 0 creates single-line paragraphs with'
	SAY ' no word wrapping; good for importing to a word processor program.'
	SAY ''
	SAY ' InFile and OutFile must be enclosed with quotation marks if the filenames'
	SAY ' contain spaces.'
	SAY ''
	SAY LGre||'Available Options:'||Non
	SAY '  /F = specify output format as TEXT or HTML'
	SAY LWhi||'       Default is TEXT if not given.'||Non
	SAY '  /E = specify output EOL character'
	SAY '       To output for OS/2, DOS or Win, use /E DOS or /E CRLF or /E 0d0a'
	SAY '       To output for UNIX, use /E UNIX or /E LF or /E 0a'
	SAY '       To output for Macintosh, use /E MAC or /E CR or /E 0d'
	SAY LWhi||'       Default is DOS if not given.'||Non
	SAY '  /? = show this help screen'

EXIT


Prep:

	X = Lastpos('.', Origfile)
	IF X > 0 THEN
		DO
			Tempfile = Substr(Origfile, 1, X-1)||'.tmp'
			IF OutFile = '' THEN
				Outfile = Substr(Origfile, 1, X-1)||'.out'
		END
	ELSE
		DO
			Tempfile = Origfile||'.tmp'
			IF OutFile = '' THEN
				Outfile = Origfile||'.out'
		END

	/* Backup original file to temporary directory for safety */
	SAY LBlu||'*' LTur||'Backing up' LBro||Origfile LTur||'to /temp directory'||Non
	CALL SysMkDir 'temp'
	'@copy "'||Origfile||'" temp'

RETURN


Reformat:

	/* Remove extraneous spaces from input file and concatenate lines */
	SAY LBlu||'*' LTur||'Reformatting' LBro||Origfile LTur||'to' LBro||Tempfile||Non

	CALL SysFileDelete TempFile

	DO WHILE Lines(Origfile) > 0
		Inline = Linein(Origfile,,1)
		DO FOREVER
			X = Lastpos('09'x, Inline)
			IF X > 0 THEN
				DO
					A = substr(Inline, 1, X-1)
					B = substr(Inline, X+1)
					Inline = A||' '||B
				END
			ELSE
				LEAVE
		END
		Templine = Space(Inline)
		IF Translate(Left(Templine, 3)) = '<P>' THEN
			Templine = Substr(Templine, 4)
		Outline = Outline||' '||Templine
	END
	CALL Lineout(Origfile)

	/* Substitute double linefeeds for double spaces in concatenated line */
	DO FOREVER
		X = Lastpos('  ',Outline)
		IF X > 0 THEN
			DO
				A = Substr(Outline, 1, X - 1)
				B = Substr(Outline, X + 2)
				IF OutForm = 'HTML' THEN
					Outline = A||EOL||EOL||'<P>'||B
				ELSE
					Outline = A||EOL||EOL||B
			END
		ELSE
			LEAVE
	END
	Outline = Space(Outline)
	IF OutForm = 'HTML' THEN
		Outline = '<P>'||Outline
	CALL Lineout Tempfile, Outline
	CALL Lineout(Tempfile)

RETURN


Wrap:

	CALL SysFileDelete OutFile
	OutLine = ''
	/* Wrap single line paragraphs to specified margin width */
	SAY LBlu||'*' LTur||'Word wrapping' LBro||Tempfile LTur||'to' LBro||Outfile||Non
	DO WHILE Lines(Tempfile) > 0
		Inline = Linein(Tempfile,,1)
		IF Length(Inline) > Margin THEN
			DO
				Start = 1
				DO UNTIL Start >= Length(Inline)
					A = Substr(Inline, Start, Margin + 1)
					Blank = Lastpos(' ', A)
					IF Blank = 0 THEN
						DO
							Inline = Inline||' '
							Blank = Pos(' ', Inline, Start) - Start + 1
						END
					Last = Blank - 1
					A = Substr(Inline, Start, Last)
					A = Space(A)
					OutLine = OutLine||A||EOL
					Start = Start + Blank
				END
			END
		ELSE
			OutLine = OutLine||InLine||EOL
	END
	CALL Lineout OutFile, OutLine
	CALL Lineout(Tempfile)
	CALL Lineout(Outfile)

RETURN


CleanUp:

	IF Margin = 0 THEN
		DO
			SAY LBlu||'*' LTur||'Renaming' LBro||Tempfile LTur||'to' LBro||Outfile||Non
			'@ren "'||TempFile||'" "'||OutFile||'"'
		END

	SAY LBlu||'*' LTur||'Cleaning up directory'||Non
	CALL SysFileDelete Tempfile

RETURN

